@@ -4,9 +4,8 @@
  * This file is released under the GPL.
  */
 
-#include <linux/fs.h>
-
 #include "dm.h"
+
 #include <linux/dm-ioctl.h>
 
 static void free_params(struct dm_ioctl *p)
@@ -35,31 +34,31 @@ static int copy_params(struct dm_ioctl *user, struct dm_ioctl **result)
  * check a string doesn't overrun the chunk of
  * memory we copied from userland.
  */
-static int valid_str(char *str, void *end)
+static int valid_str(char *str, void *begin, void *end)
 {
-	while (((void *) str < end) && *str)
+	while (((void *) str >= begin) && ((void *) str < end) && *str)
 		str++;
 
 	return *str ? 0 : 1;
 }
 
-static int first_target(struct dm_ioctl *a, void *end,
+static int first_target(struct dm_ioctl *a, void *begin, void *end,
 			struct dm_target_spec **spec, char **params)
 {
 	*spec = (struct dm_target_spec *) (a + 1);
 	*params = (char *) (*spec + 1);
 
-	return valid_str(*params, end);
+	return valid_str(*params, begin, end);
 }
 
-static int next_target(struct dm_target_spec *last, void *end,
+static int next_target(struct dm_target_spec *last, void *begin, void *end,
 		       struct dm_target_spec **spec, char **params)
 {
 	*spec = (struct dm_target_spec *)
 	    (((unsigned char *) last) + last->next);
 	*params = (char *) (*spec + 1);
 
-	return valid_str(*params, end);
+	return valid_str(*params, begin, end);
 }
 
 void dm_error(const char *message)
@@ -84,11 +83,11 @@ static int gap(struct dm_table *table, struct dm_target_spec *spec)
 
 static int populate_table(struct dm_table *table, struct dm_ioctl *args)
 {
-	int i = 0, r, first = 1;
+	int i = 0, r, first = 1, argc;
 	struct dm_target_spec *spec;
-	char *params;
+	char *params, *argv[MAX_ARGS];
 	struct target_type *ttype;
-	void *context, *end;
+	void *context, *begin, *end;
 	offset_t high = 0;
 
 	if (!args->target_count) {
@@ -96,33 +95,39 @@ static int populate_table(struct dm_table *table, struct dm_ioctl *args)
 		return -EINVAL;
 	}
 
-	end = ((void *) args) + args->data_size;
+	begin = (void *) args;
+	end = begin + args->data_size;
 
 #define PARSE_ERROR(msg) {dm_error(msg); return -EINVAL;}
 
 	for (i = 0; i < args->target_count; i++) {
 
-		r = first ? first_target(args, end, &spec, &params) :
-		    next_target(spec, end, &spec, &params);
+		r = first ? first_target(args, begin, end, &spec, &params) :
+		    next_target(spec, begin, end, &spec, &params);
 
 		if (!r)
 			PARSE_ERROR("unable to find target");
 
-		/* lookup the target type */
-		if (!(ttype = dm_get_target_type(spec->target_type)))
+		/* Look up the target type */
+		ttype = dm_get_target_type(spec->target_type);
+		if (!ttype)
 			PARSE_ERROR("unable to find target type");
 
 		if (gap(table, spec))
 			PARSE_ERROR("gap in target ranges");
 
-		/* build the target */
-		if (ttype->ctr(table, spec->sector_start, spec->length, params,
-			       &context))  {
+		/* Split up the parameter list */
+		if (split_args(MAX_ARGS, &argc, argv, params) < 0)
+			PARSE_ERROR("Too many arguments");
+
+		/* Build the target */
+		if (ttype->ctr(table, spec->sector_start, spec->length,
+			       argc, argv, &context))  {
 			dm_error(context);
 			PARSE_ERROR("target constructor failed");
 		}
 
-		/* add the target to the table */
+		/* Add the target to the table */
 		high = spec->sector_start + (spec->length - 1);
 		if (dm_table_add_target(table, high, ttype, context))
 			PARSE_ERROR("internal error adding target to table");
